#options( show.error.messages=F, error = function () { cat( geterrmessage(), file=stderr() ); q( "no", 1, F ) } )

args <- commandArgs(trailingOnly = TRUE)

inFile = args[1]
outFile = args[2]
outDir = args[3]
clonalType = args[4]
species = args[5]
locus = args[6]
selection = args[7]



if (!("gridExtra" %in% rownames(installed.packages()))) {
	install.packages("gridExtra", repos="http://cran.xl-mirror.nl/") 
}
library(gridExtra)
if (!("ggplot2" %in% rownames(installed.packages()))) {
	install.packages("ggplot2", repos="http://cran.xl-mirror.nl/") 
}
library(ggplot2)
if (!("plyr" %in% rownames(installed.packages()))) {
	install.packages("plyr", repos="http://cran.xl-mirror.nl/") 
}			
library(plyr)

if (!("data.table" %in% rownames(installed.packages()))) {
	install.packages("data.table", repos="http://cran.xl-mirror.nl/") 
}
library(data.table)

if (!("reshape2" %in% rownames(installed.packages()))) {
	install.packages("reshape2", repos="http://cran.xl-mirror.nl/") 
}
library(reshape2)


test = read.table(inFile, sep="\t", header=TRUE, fill=T, comment.char="")

test = test[test$Sample != "",]

test$Top.V.Gene = gsub("[*]([0-9]+)", "", test$Top.V.Gene)
test$Top.D.Gene = gsub("[*]([0-9]+)", "", test$Top.D.Gene)
test$Top.J.Gene = gsub("[*]([0-9]+)", "", test$Top.J.Gene)

#test$VDJCDR3 = do.call(paste, c(test[c("Top.V.Gene", "Top.D.Gene", "Top.J.Gene","CDR3.Seq.DNA")], sep = ":"))
test$VDJCDR3 = do.call(paste, c(test[unlist(strsplit(clonalType, ","))], sep = ":"))

PROD = test[test$VDJ.Frame != "In-frame with stop codon" & test$VDJ.Frame != "Out-of-frame" & test$CDR3.Found.How != "NOT_FOUND" , ]
if("Functionality" %in% colnames(test)) {
	PROD = test[test$Functionality == "productive" | test$Functionality == "productive (see comment)", ]
}

NONPROD = test[test$VDJ.Frame == "In-frame with stop codon" | test$VDJ.Frame == "Out-of-frame" | test$CDR3.Found.How == "NOT_FOUND" , ]

#PRODF = PROD[ -1]

PRODF = PROD

#PRODF = unique(PRODF)



if(selection == "unique"){
	PRODF = PRODF[!duplicated(PRODF$VDJCDR3), ]
}

PRODFV = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Sample", "Top.V.Gene")])
PRODFV$Length = as.numeric(PRODFV$Length)
Total = 0
Total = ddply(PRODFV, .(Sample), function(x) data.frame(Total = sum(x$Length)))
PRODFV = merge(PRODFV, Total, by.x='Sample', by.y='Sample', all.x=TRUE)
PRODFV = ddply(PRODFV, c("Sample", "Top.V.Gene"), summarise, relFreq= (Length*100 / Total))

PRODFD = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Sample", "Top.D.Gene")])
PRODFD$Length = as.numeric(PRODFD$Length)
Total = 0
Total = ddply(PRODFD, .(Sample), function(x) data.frame(Total = sum(x$Length)))
PRODFD = merge(PRODFD, Total, by.x='Sample', by.y='Sample', all.x=TRUE)
PRODFD = ddply(PRODFD, c("Sample", "Top.D.Gene"), summarise, relFreq= (Length*100 / Total))

PRODFJ = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Sample", "Top.J.Gene")])
PRODFJ$Length = as.numeric(PRODFJ$Length)
Total = 0
Total = ddply(PRODFJ, .(Sample), function(x) data.frame(Total = sum(x$Length)))
PRODFJ = merge(PRODFJ, Total, by.x='Sample', by.y='Sample', all.x=TRUE)
PRODFJ = ddply(PRODFJ, c("Sample", "Top.J.Gene"), summarise, relFreq= (Length*100 / Total))

V = c("v.name\tchr.orderV")
D = c("v.name\tchr.orderD")
J = c("v.name\tchr.orderJ")

if(species == "human"){
	if(locus == "igh"){		
		V = c("v.name\tchr.orderV\nIGHV3-74\t1\nIGHV3-73\t2\nIGHV3-72\t3\nIGHV2-70\t4\nIGHV1-69D\t5\nIGHV1-69-2\t6\nIGHV2-70D\t7\nIGHV1-69\t8\nIGHV3-66\t9\nIGHV3-64\t10\nIGHV4-61\t11\nIGHV4-59\t12\nIGHV1-58\t13\nIGHV3-53\t14\nIGHV5-51\t15\nIGHV3-49\t16\nIGHV3-48\t17\nIGHV1-46\t18\nIGHV1-45\t19\nIGHV3-43\t20\nIGHV4-39\t21\nIGHV3-43D\t22\nIGHV4-38-2\t23\nIGHV4-34\t24\nIGHV3-33\t25\nIGHV4-31\t26\nIGHV3-30-5\t27\nIGHV4-30-4\t28\nIGHV3-30-3\t29\nIGHV4-30-2\t30\nIGHV4-30-1\t31\nIGHV3-30\t32\nIGHV4-28\t33\nIGHV2-26\t34\nIGHV1-24\t35\nIGHV3-23D\t36\nIGHV3-23\t37\nIGHV3-21\t38\nIGHV3-20\t39\nIGHV1-18\t40\nIGHV3-15\t41\nIGHV3-13\t42\nIGHV3-11\t43\nIGHV5-10-1\t44\nIGHV3-9\t45\nIGHV1-8\t46\nIGHV3-64D\t47\nIGHV3-7\t48\nIGHV2-5\t49\nIGHV7-4-1\t50\nIGHV4-4\t51\nIGHV1-3\t52\nIGHV1-2\t53\nIGHV6-1\t54")
		D = c("v.name\tchr.orderD\nIGHD1-7\t1\nIGHD2-8\t2\nIGHD3-9\t3\nIGHD3-10\t4\nIGHD5-12\t5\nIGHD6-13\t6\nIGHD2-15\t7\nIGHD3-16\t8\nIGHD4-17\t9\nIGHD5-18\t10\nIGHD6-19\t11\nIGHD1-20\t12\nIGHD2-21\t13\nIGHD3-22\t14\nIGHD5-24\t15\nIGHD6-25\t16\nIGHD1-26\t17\nIGHD7-27\t18")
		J = c("v.name\tchr.orderJ\nIGHJ1\t1\nIGHJ2\t2\nIGHJ3\t3\nIGHJ4\t4\nIGHJ5\t5\nIGHJ6\t6")
	} else if (locus == "igk"){
		V = c("v.name\tchr.orderV\nIGKV3D-7\t1\nIGKV1D-8\t2\nIGKV1D-43\t3\nIGKV3D-11\t4\nIGKV1D-12\t5\nIGKV1D-13\t6\nIGKV3D-15\t7\nIGKV1D-16\t8\nIGKV1D-17\t9\nIGKV3D-20\t10\nIGKV2D-26\t11\nIGKV2D-28\t12\nIGKV2D-29\t13\nIGKV2D-30\t14\nIGKV1D-33\t15\nIGKV1D-39\t16\nIGKV2D-40\t17\nIGKV2-40\t18\nIGKV1-39\t19\nIGKV1-33\t20\nIGKV2-30\t21\nIGKV2-29\t22\nIGKV2-28\t23\nIGKV1-27\t24\nIGKV2-24\t25\nIGKV3-20\t26\nIGKV1-17\t27\nIGKV1-16\t28\nIGKV3-15\t29\nIGKV1-13\t30\nIGKV1-12\t31\nIGKV3-11\t32\nIGKV1-9\t33\nIGKV1-8\t34\nIGKV1-6\t35\nIGKV1-5\t36\nIGKV5-2\t37\nIGKV4-1\t38")
		D = c("v.name\tchr.orderD\n")
		J = c("v.name\tchr.orderJ\nIGKJ1\t1\nIGKJ2\t2\nIGKJ3\t3\nIGKJ4\t4\nIGKJ5\t5")
	} else if (locus == "igl"){
		V = c("v.name\tchr.orderV\nIGLV4-69\t1\nIGLV8-61\t2\nIGLV4-60\t3\nIGLV6-57\t4\nIGLV5-52\t5\nIGLV1-51\t6\nIGLV9-49\t7\nIGLV1-47\t8\nIGLV7-46\t9\nIGLV5-45\t10\nIGLV1-44\t11\nIGLV7-43\t12\nIGLV1-41\t13\nIGLV1-40\t14\nIGLV5-39\t15\nIGLV5-37\t16\nIGLV1-36\t17\nIGLV3-27\t18\nIGLV3-25\t19\nIGLV2-23\t20\nIGLV3-22\t21\nIGLV3-21\t22\nIGLV3-19\t23\nIGLV2-18\t24\nIGLV3-16\t25\nIGLV2-14\t26\nIGLV3-12\t27\nIGLV2-11\t28\nIGLV3-10\t29\nIGLV3-9\t30\nIGLV2-8\t31\nIGLV4-3\t32\nIGLV3-1\t33")
		D = c("v.name\tchr.orderD\n")
		J = c("v.name\tchr.orderJ\nIGLJ1\t1\nIGLJ2\t2\nIGLJ3\t3\nIGLJ6\t4\nIGLJ7\t5")
	}
} else if (species == "mouse"){
	if(locus == "igh"){
		cat("mouse igh not yet implemented")
	} else if (locus == "igk"){
		cat("mouse igk not yet implemented")
	} else if (locus == "igl"){
		cat("mouse igl not yet implemented")
	}
}

useD = TRUE
if(species == "human" && (locus == "igk" || locus == "igl")){
	useD = FALSE
}

tcV = textConnection(V)
Vchain = read.table(tcV, sep="\t", header=TRUE)
PRODFV = merge(PRODFV, Vchain, by.x='Top.V.Gene', by.y='v.name', all.x=TRUE)
close(tcV)

tcD = textConnection(D)
Dchain = read.table(tcD, sep="\t", header=TRUE)
PRODFD = merge(PRODFD, Dchain, by.x='Top.D.Gene', by.y='v.name', all.x=TRUE)
close(tcD)

tcJ = textConnection(J)
Jchain = read.table(tcJ, sep="\t", header=TRUE)
PRODFJ = merge(PRODFJ, Jchain, by.x='Top.J.Gene', by.y='v.name', all.x=TRUE)
close(tcJ)

setwd(outDir)

write.table(PRODF, "allUnique.csv", sep=",",quote=F,row.names=F,col.names=T)

pV = ggplot(PRODFV)
pV = pV + geom_bar( aes( x=factor(reorder(Top.V.Gene, chr.orderV)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
pV = pV + xlab("Summary of V gene") + ylab("Frequency") + ggtitle("Relative frequency of V gene usage")
write.table(x=PRODFV, file="VFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

png("VPlot.png",width = 1280, height = 720)
pV
dev.off();

if(useD){
	pD = ggplot(PRODFD)
	pD = pD + geom_bar( aes( x=factor(reorder(Top.D.Gene, chr.orderD)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
	pD = pD + xlab("Summary of D gene") + ylab("Frequency") + ggtitle("Relative frequency of D gene usage")
	write.table(x=PRODFD, file="DFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

	png("DPlot.png",width = 800, height = 600)
	print(pD)
	dev.off();
}

pJ = ggplot(PRODFJ)
pJ = pJ + geom_bar( aes( x=factor(reorder(Top.J.Gene, chr.orderJ)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
pJ = pJ + xlab("Summary of J gene") + ylab("Frequency") + ggtitle("Relative frequency of J gene usage")
write.table(x=PRODFJ, file="JFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

png("JPlot.png",width = 800, height = 600)
pJ
dev.off();

VGenes = PRODF[,c("Sample", "Top.V.Gene")]
VGenes$Top.V.Gene = gsub("-.*", "", VGenes$Top.V.Gene)
VGenes = data.frame(data.table(VGenes)[, list(Count=.N), by=c("Sample", "Top.V.Gene")])
TotalPerSample = data.frame(data.table(VGenes)[, list(total=sum(.SD$Count)), by=Sample])
VGenes = merge(VGenes, TotalPerSample, by="Sample")
VGenes$Frequency = VGenes$Count * 100 / VGenes$total
VPlot = ggplot(VGenes)
VPlot = VPlot + geom_bar(aes( x = Top.V.Gene, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
				ggtitle("Distribution of V gene families") + 
				ylab("Percentage of sequences")
png("VFPlot.png")
VPlot
dev.off();
write.table(x=VGenes, file="VFFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

if(useD){
	DGenes = PRODF[,c("Sample", "Top.D.Gene")]
	DGenes$Top.D.Gene = gsub("-.*", "", DGenes$Top.D.Gene)
	DGenes = data.frame(data.table(DGenes)[, list(Count=.N), by=c("Sample", "Top.D.Gene")])
	TotalPerSample = data.frame(data.table(DGenes)[, list(total=sum(.SD$Count)), by=Sample])
	DGenes = merge(DGenes, TotalPerSample, by="Sample")
	DGenes$Frequency = DGenes$Count * 100 / DGenes$total
	DPlot = ggplot(DGenes)
	DPlot = DPlot + geom_bar(aes( x = Top.D.Gene, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
					ggtitle("Distribution of D gene families") + 
					ylab("Percentage of sequences")
	png("DFPlot.png")
	print(DPlot)
	dev.off();
	write.table(x=DGenes, file="DFFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)
}

JGenes = PRODF[,c("Sample", "Top.J.Gene")]
JGenes$Top.J.Gene = gsub("-.*", "", JGenes$Top.J.Gene)
JGenes = data.frame(data.table(JGenes)[, list(Count=.N), by=c("Sample", "Top.J.Gene")])
TotalPerSample = data.frame(data.table(JGenes)[, list(total=sum(.SD$Count)), by=Sample])
JGenes = merge(JGenes, TotalPerSample, by="Sample")
JGenes$Frequency = JGenes$Count * 100 / JGenes$total
JPlot = ggplot(JGenes)
JPlot = JPlot + geom_bar(aes( x = Top.J.Gene, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
				ggtitle("Distribution of J gene families") + 
				ylab("Percentage of sequences")
png("JFPlot.png")
JPlot
dev.off();
write.table(x=JGenes, file="JFFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

CDR3Length = data.frame(data.table(PRODF)[, list(Count=.N), by=c("Sample", "CDR3.Length.DNA")])
TotalPerSample = data.frame(data.table(CDR3Length)[, list(total=sum(.SD$Count)), by=Sample])
CDR3Length = merge(CDR3Length, TotalPerSample, by="Sample")
CDR3Length$Frequency = CDR3Length$Count * 100 / CDR3Length$total
CDR3LengthPlot = ggplot(CDR3Length)
CDR3LengthPlot = CDR3LengthPlot + geom_bar(aes( x = CDR3.Length.DNA, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
				ggtitle("Length distribution of CDR3") + 
				xlab("CDR3 Length") + 
				ylab("Percentage of sequences")
png("CDR3LengthPlot.png",width = 1280, height = 720)
CDR3LengthPlot
dev.off()
write.table(x=CDR3Length, file="CDR3LengthPlot.csv", sep=",",quote=F,row.names=F,col.names=T)

revVchain = Vchain
revDchain = Dchain
revVchain$chr.orderV = rev(revVchain$chr.orderV)
revDchain$chr.orderD = rev(revDchain$chr.orderD)

if(useD){
	plotVD <- function(dat){
		if(length(dat[,1]) == 0){
			return()
		}
		img = ggplot() + 
		geom_tile(data=dat, aes(x=factor(reorder(Top.D.Gene, chr.orderD)), y=factor(reorder(Top.V.Gene, chr.orderV)), fill=relLength)) + 
		theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
		scale_fill_gradient(low="gold", high="blue", na.value="white") + 
		ggtitle(paste(unique(dat$Sample), " (N=" , sum(dat$Length, na.rm=T) ,")", sep="")) + 
		xlab("D genes") + 
		ylab("V Genes")
		
		png(paste("HeatmapVD_", unique(dat[3])[1,1] , ".png", sep=""), width=150+(15*length(Dchain$v.name)), height=100+(15*length(Vchain$v.name)))
		print(img)
		dev.off()
		write.table(x=acast(dat, Top.V.Gene~Top.D.Gene, value.var="Length"), file=paste("HeatmapVD_", unique(dat[3])[1,1], ".csv", sep=""), sep=",",quote=F,row.names=T,col.names=NA)
	}

	VandDCount = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Top.V.Gene", "Top.D.Gene", "Sample")])

	VandDCount$l = log(VandDCount$Length)
	maxVD = data.frame(data.table(VandDCount)[, list(max=max(l)), by=c("Sample")])
	VandDCount = merge(VandDCount, maxVD, by.x="Sample", by.y="Sample", all.x=T)
	VandDCount$relLength = VandDCount$l / VandDCount$max

	cartegianProductVD = expand.grid(Top.V.Gene = Vchain$v.name, Top.D.Gene = Dchain$v.name, Sample = unique(test$Sample))

	completeVD = merge(VandDCount, cartegianProductVD, all.y=TRUE)
	completeVD = merge(completeVD, revVchain, by.x="Top.V.Gene", by.y="v.name", all.x=TRUE)
	completeVD = merge(completeVD, Dchain, by.x="Top.D.Gene", by.y="v.name", all.x=TRUE)
	VDList = split(completeVD, f=completeVD[,"Sample"])

	lapply(VDList, FUN=plotVD)
}

plotVJ <- function(dat){
	if(length(dat[,1]) == 0){
		return()
	}
	cat(paste(unique(dat[3])[1,1]))
	img = ggplot() + 
	geom_tile(data=dat, aes(x=factor(reorder(Top.J.Gene, chr.orderJ)), y=factor(reorder(Top.V.Gene, chr.orderV)), fill=relLength)) + 
	theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
	scale_fill_gradient(low="gold", high="blue", na.value="white") + 
	ggtitle(paste(unique(dat$Sample), " (N=" , sum(dat$Length, na.rm=T) ,")", sep="")) + 
	xlab("J genes") + 
	ylab("V Genes")
	
	png(paste("HeatmapVJ_", unique(dat[3])[1,1] , ".png", sep=""), width=150+(15*length(Jchain$v.name)), height=100+(15*length(Vchain$v.name)))
	print(img)
	dev.off()
	write.table(x=acast(dat, Top.V.Gene~Top.J.Gene, value.var="Length"), file=paste("HeatmapVJ_", unique(dat[3])[1,1], ".csv", sep=""), sep=",",quote=F,row.names=T,col.names=NA)
}

VandJCount = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Top.V.Gene", "Top.J.Gene", "Sample")])

VandJCount$l = log(VandJCount$Length)
maxVJ = data.frame(data.table(VandJCount)[, list(max=max(l)), by=c("Sample")])
VandJCount = merge(VandJCount, maxVJ, by.x="Sample", by.y="Sample", all.x=T)
VandJCount$relLength = VandJCount$l / VandJCount$max

cartegianProductVJ = expand.grid(Top.V.Gene = Vchain$v.name, Top.J.Gene = Jchain$v.name, Sample = unique(test$Sample))

completeVJ = merge(VandJCount, cartegianProductVJ, all.y=TRUE)
completeVJ = merge(completeVJ, revVchain, by.x="Top.V.Gene", by.y="v.name", all.x=TRUE)
completeVJ = merge(completeVJ, Jchain, by.x="Top.J.Gene", by.y="v.name", all.x=TRUE)
VJList = split(completeVJ, f=completeVJ[,"Sample"])
lapply(VJList, FUN=plotVJ)

if(useD){
	plotDJ <- function(dat){
		if(length(dat[,1]) == 0){
			return()
		}
		img = ggplot() + 
		geom_tile(data=dat, aes(x=factor(reorder(Top.J.Gene, chr.orderJ)), y=factor(reorder(Top.D.Gene, chr.orderD)), fill=relLength)) + 
		theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
		scale_fill_gradient(low="gold", high="blue", na.value="white") + 
		ggtitle(paste(unique(dat$Sample), " (N=" , sum(dat$Length, na.rm=T) ,")", sep="")) + 
		xlab("J genes") + 
		ylab("D Genes")
		
		png(paste("HeatmapDJ_", unique(dat[3])[1,1] , ".png", sep=""), width=150+(15*length(Jchain$v.name)), height=100+(15*length(Dchain$v.name)))
		print(img)
		dev.off()
		write.table(x=acast(dat, Top.D.Gene~Top.J.Gene, value.var="Length"), file=paste("HeatmapDJ_", unique(dat[3])[1,1], ".csv", sep=""), sep=",",quote=F,row.names=T,col.names=NA)
	}


	DandJCount = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Top.D.Gene", "Top.J.Gene", "Sample")])

	DandJCount$l = log(DandJCount$Length)
	maxDJ = data.frame(data.table(DandJCount)[, list(max=max(l)), by=c("Sample")])
	DandJCount = merge(DandJCount, maxDJ, by.x="Sample", by.y="Sample", all.x=T)
	DandJCount$relLength = DandJCount$l / DandJCount$max

	cartegianProductDJ = expand.grid(Top.D.Gene = Dchain$v.name, Top.J.Gene = Jchain$v.name, Sample = unique(test$Sample))

	completeDJ = merge(DandJCount, cartegianProductDJ, all.y=TRUE)
	completeDJ = merge(completeDJ, revDchain, by.x="Top.D.Gene", by.y="v.name", all.x=TRUE)
	completeDJ = merge(completeDJ, Jchain, by.x="Top.J.Gene", by.y="v.name", all.x=TRUE)
	DJList = split(completeDJ, f=completeDJ[,"Sample"])
	lapply(DJList, FUN=plotDJ)
}

sampleFile <- file("samples.txt")
un = unique(test$Sample)
un = paste(un, sep="\n")
writeLines(un, sampleFile)
close(sampleFile)


if("Replicate" %in% colnames(test))
{
	clonalityFrame = PROD
	clonalityFrame$ReplicateConcat = do.call(paste, c(clonalityFrame[c("VDJCDR3", "Sample", "Replicate")], sep = ":"))
	clonalityFrame = clonalityFrame[!duplicated(clonalityFrame$ReplicateConcat), ]
	write.table(clonalityFrame, "clonalityComplete.csv", sep=",",quote=F,row.names=F,col.names=T)

	ClonalitySampleReplicatePrint <- function(dat){
	    write.table(dat, paste("clonality_", unique(dat$Sample) , "_", unique(dat$Replicate), ".csv", sep=""), sep=",",quote=F,row.names=F,col.names=T)
	}

    clonalityFrameSplit = split(clonalityFrame, f=clonalityFrame[,c("Sample", "Replicate")])
    #lapply(clonalityFrameSplit, FUN=ClonalitySampleReplicatePrint)

    ClonalitySamplePrint <- function(dat){
	    write.table(dat, paste("clonality_", unique(dat$Sample) , ".csv", sep=""), sep=",",quote=F,row.names=F,col.names=T)
	}

    clonalityFrameSplit = split(clonalityFrame, f=clonalityFrame[,"Sample"])
    #lapply(clonalityFrameSplit, FUN=ClonalitySamplePrint)

	clonalFreq = data.frame(data.table(clonalityFrame)[, list(Type=.N), by=c("Sample", "VDJCDR3")])
	clonalFreqCount = data.frame(data.table(clonalFreq)[, list(Count=.N), by=c("Sample", "Type")])
	clonalFreqCount$realCount = clonalFreqCount$Type * clonalFreqCount$Count
	clonalSum = data.frame(data.table(clonalFreqCount)[, list(Reads=sum(realCount)), by=c("Sample")])
	clonalFreqCount = merge(clonalFreqCount, clonalSum, by.x="Sample", by.y="Sample")

	ct = c('Type\tWeight\n2\t1\n3\t3\n4\t6\n5\t10\n6\t15')
	tcct = textConnection(ct)
	CT  = read.table(tcct, sep="\t", header=TRUE)
	close(tcct)
	clonalFreqCount = merge(clonalFreqCount, CT, by.x="Type", by.y="Type", all.x=T)
	clonalFreqCount$WeightedCount = clonalFreqCount$Count * clonalFreqCount$Weight

	ReplicateReads = data.frame(data.table(clonalityFrame)[, list(Type=.N), by=c("Sample", "Replicate", "VDJCDR3")])
	ReplicateReads = data.frame(data.table(ReplicateReads)[, list(Reads=.N), by=c("Sample", "Replicate")])
	clonalFreqCount$Reads = as.numeric(clonalFreqCount$Reads)
	ReplicateReads$squared = ReplicateReads$Reads * ReplicateReads$Reads

	ReplicatePrint <- function(dat){
		write.table(dat[-1], paste("ReplicateReads_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
	}

	ReplicateSplit = split(ReplicateReads, f=ReplicateReads[,"Sample"])
	lapply(ReplicateSplit, FUN=ReplicatePrint)

	ReplicateReads = data.frame(data.table(ReplicateReads)[, list(ReadsSum=sum(Reads), ReadsSquaredSum=sum(squared)), by=c("Sample")])
	clonalFreqCount = merge(clonalFreqCount, ReplicateReads, by.x="Sample", by.y="Sample", all.x=T)


	ReplicateSumPrint <- function(dat){
		write.table(dat[-1], paste("ReplicateSumReads_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
	}

	ReplicateSumSplit = split(ReplicateReads, f=ReplicateReads[,"Sample"])
	lapply(ReplicateSumSplit, FUN=ReplicateSumPrint)

	clonalFreqCountSum = data.frame(data.table(clonalFreqCount)[, list(Numerator=sum(WeightedCount, na.rm=T)), by=c("Sample")])
	clonalFreqCount = merge(clonalFreqCount, clonalFreqCountSum, by.x="Sample", by.y="Sample", all.x=T)
	clonalFreqCount$ReadsSum = as.numeric(clonalFreqCount$ReadsSum) #prevent integer overflow
	clonalFreqCount$Denominator = (((clonalFreqCount$ReadsSum * clonalFreqCount$ReadsSum) - clonalFreqCount$ReadsSquaredSum) / 2)
	clonalFreqCount$Result = (clonalFreqCount$Numerator + 1) / (clonalFreqCount$Denominator + 1)

	ClonalityScorePrint <- function(dat){
		write.table(dat$Result, paste("ClonalityScore_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
	}

	clonalityScore = clonalFreqCount[c("Sample", "Result")]
	clonalityScore = unique(clonalityScore)

	clonalityScoreSplit = split(clonalityScore, f=clonalityScore[,"Sample"])
	lapply(clonalityScoreSplit, FUN=ClonalityScorePrint)

	clonalityOverview = clonalFreqCount[c("Sample", "Type", "Count", "Weight", "WeightedCount")]



	ClonalityOverviewPrint <- function(dat){
		write.table(dat[-1], paste("ClonalityOverView_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
	}

	clonalityOverviewSplit = split(clonalityOverview, f=clonalityOverview$Sample)
	lapply(clonalityOverviewSplit, FUN=ClonalityOverviewPrint)
}

if("Functionality" %in% colnames(test))
{
	newData = data.frame(data.table(PROD)[,list(unique=.N, 
				VH.DEL=mean(X3V.REGION.trimmed.nt.nb),
				P1=mean(P3V.nt.nb),
				N1=mean(N1.REGION.nt.nb),
				P2=mean(P5D.nt.nb),
				DEL.DH=mean(X5D.REGION.trimmed.nt.nb),
				DH.DEL=mean(X3D.REGION.trimmed.nt.nb),
				P3=mean(P3D.nt.nb),
				N2=mean(N2.REGION.nt.nb),
				P4=mean(P5J.nt.nb),
				DEL.JH=mean(X5J.REGION.trimmed.nt.nb),
				Total.Del=(	mean(X3V.REGION.trimmed.nt.nb) + 
							mean(X5D.REGION.trimmed.nt.nb) + 
							mean(X3D.REGION.trimmed.nt.nb) +
							mean(X5J.REGION.trimmed.nt.nb)),
							
				Total.N=(	mean(N1.REGION.nt.nb) +
							mean(N2.REGION.nt.nb)),
							
				Total.P=(	mean(P3V.nt.nb) +
							mean(P5D.nt.nb) +
							mean(P3D.nt.nb) +
							mean(P5J.nt.nb))),
				by=c("Sample")])
	write.table(newData, "junctionAnalysis.csv" , sep=",",quote=F,na="-",row.names=F,col.names=F)
}
